'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-a057b2d4.js');
const fetcher = require('./fetcher-574b1e78.js');

function formatTemplate(params) {
  const { template, data } = params;
  const _template = String(template);
  if (_template.includes('$count') && data.count == null) {
    console.warn('$count is present but `count` variable is missing');
  }
  return _template.replace('$count', data.count);
}

function formatShortNumber(number) {
  const _number = Number(number);
  if (Number.isNaN(_number)) {
    return 'Invalid number';
  }
  if (_number < 1000) {
    return _number;
  }
  if (_number < 1000000) {
    return `${(_number / 1000).toFixed(2).replace('.', ',')}k`;
  }
  return `${(_number / 1000000).toFixed(2).replace('.', ',')}mln`;
}

const hrWidgetCss = ":host{display:block}";

const HrWidget = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.reaction = index.createEvent(this, "reaction", 7);
    this.reactionsLimitReached = index.createEvent(this, "reactionsLimitReached", 7);
    /**
     * Form element
     */
    this.form = document.querySelector('hr-form');
    this.token = undefined;
    this.resource = undefined;
    this.classes = undefined;
    this.url = fetcher.DEFAULT_URL;
    this.response = {
      status: 'loading',
      data: null
    };
  }
  _isFirstReaction() {
    var _a;
    return !((_a = this.response.data.reacted) === null || _a === void 0 ? void 0 : _a.id);
  }
  _isUnlimitedReactions() {
    return this.response.data.preferences.allowMultiple && this.response.data.preferences.maxReactions === 0;
  }
  _isLimitReactionsNotReached(reaction) {
    return this.response.data.reacted.id === reaction.id &&
      this.response.data.preferences.allowMultiple &&
      this.response.data.reacted.count < this.response.data.preferences.maxReactions;
  }
  _isLimitReactionsReached() {
    return this.response.data.reacted.count >= this.response.data.preferences.maxReactions;
  }
  _getAffiliateURL() {
    if (typeof window !== 'undefined' && 'location' in window) {
      return `https://happyreact.com/?utm_source=${window.location.origin}&utm_medium=widget&utm_campaign=footer`;
    }
    return `https://happyreact.com/?utm_source=feedback&utm_medium=widget&utm_campaign=footer`;
  }
  async collect(reaction) {
    if (this.response.status !== 'success') {
      return;
    }
    const data = structuredClone(this.response.data);
    if (this._isFirstReaction() ||
      this._isUnlimitedReactions() ||
      this._isLimitReactionsNotReached(reaction)) {
      const foundReaction = data.reactions.find((currentReaction) => currentReaction.id === reaction.id);
      if (!data.reacted.id) {
        data.reacted.id = reaction.id;
        data.reacted.count = 1;
      }
      else {
        data.reacted.count += 1;
      }
      foundReaction.count += 1;
      this.response = {
        status: 'success',
        data
      };
      const response = await fetcher.collectReaction({
        token: this.token,
        resource: this.resource,
        reactionId: reaction.id,
        url: this.url,
        count: 1
      });
      this.onReactionHandler(reaction, { resources: response.resources });
    }
    else if (this._isLimitReactionsReached()) {
      this.onReactionsLimitReachedHandler();
    }
  }
  /**
   * onReaction event handler
   * @param reaction
   */
  onReactionHandler(reaction, data) {
    this.reaction.emit(reaction);
    if (this.form) {
      this.form.dispatchEvent(new CustomEvent('show', {
        detail: {
          token: this.token,
          reactionId: reaction.id,
          url: this.url,
          resources: data.resources,
          count: reaction.count
        }
      }));
    }
  }
  /**
   * onReactionsLimitReached event handler
   */
  onReactionsLimitReachedHandler() {
    this.reactionsLimitReached.emit();
  }
  componentWillLoad() {
    fetcher.fetchProject({
      token: this.token,
      resource: this.resource,
      url: this.url
    })
      .then((response) => {
      this.response = {
        status: 'success',
        data: response
      };
    })
      .catch((cause) => {
      this.response = {
        status: 'error',
        data: cause.message
      };
    });
  }
  render() {
    var _a;
    const getClass = fetcher.createGetClass(this.classes);
    return (index.h(index.Host, { class: getClass('root') }, index.h("div", { class: fetcher.clsx('hr-container', getClass('container')) }, index.h(fetcher.Fetcher, { status: this.response.status }, index.h("div", { class: fetcher.clsx('hr-grid', getClass('grid')) }, (_a = this.response.data) === null || _a === void 0 ? void 0 : _a.reactions.map((reaction) => (index.h("div", { class: fetcher.clsx('hr-cell', getClass('cell')), key: reaction.id }, index.h("button", { onClick: () => this.collect(reaction), class: fetcher.clsx('hr-reaction', getClass('reaction'), {
        [fetcher.clsx('hr-reaction-active', getClass('reactionActive'))]: this.response.data.reacted.id === reaction.id
      }) }, index.h("span", { class: fetcher.clsx('hr-reaction-icon', getClass('reactionIcon')) }, reaction.icon), reaction.label && (index.h("span", { class: fetcher.clsx('hr-reaction-text', getClass('reactionText')) }, formatTemplate({
      template: reaction.label,
      data: {
        count: this.response.data.preferences.shortNumber
          ? String(formatShortNumber(reaction.count))
          : String(reaction.count)
      }
    })))))))), index.h("a", { href: this._getAffiliateURL(), rel: "noopener", target: "_blank", class: "hr-powered-by", ref: (node) => {
        var _a;
        if (node) {
          node.style.setProperty('display', !((_a = this.response.data) === null || _a === void 0 ? void 0 : _a.preferences.whitelabel) ? 'block' : 'none', 'important');
        }
      } }, "Feedback widget by Happy React")))));
  }
};
HrWidget.style = hrWidgetCss;

exports.hr_widget = HrWidget;

//# sourceMappingURL=hr-widget.cjs.entry.js.map