import { h, proxyCustomElement, HTMLElement, createEvent, Host, Fragment } from '@stencil/core/internal/client';
import { c as clsx, s as saveResponse, f as fetchForm, F as Fetcher, a as createGetClass } from './fetcher.js';

const TextInput = (props) => {
  const { id, name, label, hint, error, key, onInput, getClass } = props;
  return (h("div", { class: clsx("hr-field", getClass('field')), key: key },
    h("label", { class: clsx("hr-label", getClass('label')) },
      h("div", { class: clsx("hr-label-text", getClass('labelText')) }, label)),
    h("input", { id: id, type: "text", name: name, class: clsx("hr-input hr-input-text", getClass('input'), getClass('inputText')), onInput: onInput }),
    hint && (h("div", { class: clsx("hr-input-hint", getClass('inputHint')) }, hint)),
    error && (h("div", { class: clsx("hr-input-error", getClass('inputError')) }, error))));
};

const ContentInput = (props) => {
  // const { name, label, hint, error, key, onInput } = props;
  return (h("div", { class: "hr-input-content", key: props.key }));
};

const TextareaInput = (props) => {
  const { name, label, hint, error, key, onInput, getClass } = props;
  return (h("div", { class: clsx("hr-field", getClass('field')), key: key },
    h("label", { class: clsx("hr-label", getClass('label')) },
      h("div", { class: clsx("hr-label-text", getClass('labelText')) }, label)),
    h("textarea", { name: name, class: clsx("hr-input hr-input-textarea", getClass('input'), getClass('inputTextarea')), onInput: onInput }),
    hint && (h("div", { class: clsx("hr-input-hint", getClass('inputHint')) }, hint)),
    error && (h("div", { class: clsx("hr-input-error", getClass('inputError')) }, error))));
};

const Success = (props) => {
  const { icon, content, getClass, } = props;
  return (h("div", { class: clsx("hr-success", getClass('success')) },
    h("div", { class: clsx("hr-success-icon", getClass('successIcon')) }, icon),
    h("div", { class: clsx("hr-success-content", getClass('successContent')) }, content)));
};

var e={"":["<em>","</em>"],_:["<strong>","</strong>"],"*":["<strong>","</strong>"],"~":["<s>","</s>"],"\n":["<br />"]," ":["<br />"],"-":["<hr />"]};function n(e){return e.replace(RegExp("^"+(e.match(/^(\t| )+/)||"")[0],"gm"),"")}function r(e){return (e+"").replace(/"/g,"&quot;").replace(/</g,"&lt;").replace(/>/g,"&gt;")}function t(a,c){var o,l,g,s,p,u=/((?:^|\n+)(?:\n---+|\* \*(?: \*)+)\n)|(?:^``` *(\w*)\n([\s\S]*?)\n```$)|((?:(?:^|\n+)(?:\t|  {2,}).+)+\n*)|((?:(?:^|\n)([>*+-]|\d+\.)\s+.*)+)|(?:!\[([^\]]*?)\]\(([^)]+?)\))|(\[)|(\](?:\(([^)]+?)\))?)|(?:(?:^|\n+)([^\s].*)\n(-{3,}|={3,})(?:\n+|$))|(?:(?:^|\n+)(#{1,6})\s*(.+)(?:\n+|$))|(?:`([^`].*?)`)|(  \n\n*|\n{2,}|__|\*\*|[_*]|~~)/gm,m=[],h="",i=c||{},d=0;function f(n){var r=e[n[1]||""],t=m[m.length-1]==n;return r?r[1]?(t?m.pop():m.push(n),r[0|t]):r[0]:n}function $(){for(var e="";m.length;)e+=f(m[m.length-1]);return e}for(a=a.replace(/^\[(.+?)\]:\s*(.+)$/gm,function(e,n,r){return i[n.toLowerCase()]=r,""}).replace(/^\n+|\n+$/g,"");g=u.exec(a);)l=a.substring(d,g.index),d=u.lastIndex,o=g[0],l.match(/[^\\](\\\\)*\\$/)||((p=g[3]||g[4])?o='<pre class="code '+(g[4]?"poetry":g[2].toLowerCase())+'"><code'+(g[2]?' class="language-'+g[2].toLowerCase()+'"':"")+">"+n(r(p).replace(/^\n+|\n+$/g,""))+"</code></pre>":(p=g[6])?(p.match(/\./)&&(g[5]=g[5].replace(/^\d+/gm,"")),s=t(n(g[5].replace(/^\s*[>*+.-]/gm,""))),">"==p?p="blockquote":(p=p.match(/\./)?"ol":"ul",s=s.replace(/^(.*)(\n|$)/gm,"<li>$1</li>")),o="<"+p+">"+s+"</"+p+">"):g[8]?o='<img src="'+r(g[8])+'" alt="'+r(g[7])+'">':g[10]?(h=h.replace("<a>",'<a href="'+r(g[11]||i[l.toLowerCase()])+'">'),o=$()+"</a>"):g[9]?o="<a>":g[12]||g[14]?o="<"+(p="h"+(g[14]?g[14].length:g[13]>"="?1:2))+">"+t(g[12]||g[15],i)+"</"+p+">":g[16]?o="<code>"+r(g[16])+"</code>":(g[17]||g[1])&&(o=f(g[17]||"--"))),h+=l,h+=o;return (h+a.substring(d)+$()).replace(/^\n+|\n+$/g,"")}

var escapes = {
  '&': '&amp;',
  '<': '&lt;',
  '>': '&gt;',
  '"': '&quot;',
  "'": '&#39;'
};
var unescapes = {
  '&amp;': '&',
  '&lt;': '<',
  '&gt;': '>',
  '&quot;': '"',
  '&#39;': "'"
};
var rescaped = /(&amp;|&lt;|&gt;|&quot;|&#39;)/g;
var runescaped = /[&<>"']/g;

function escapeHtmlChar (match) {
  return escapes[match];
}
function unescapeHtmlChar (match) {
  return unescapes[match];
}

function escapeHtml (text) {
  return text == null ? '' : String(text).replace(runescaped, escapeHtmlChar);
}

function unescapeHtml (html) {
  return html == null ? '' : String(html).replace(rescaped, unescapeHtmlChar);
}

escapeHtml.options = unescapeHtml.options = {};

var she = {
  encode: escapeHtml,
  escape: escapeHtml,
  decode: unescapeHtml,
  unescape: unescapeHtml,
  version: '1.0.0-browser'
};

function assignment (result) {
  var stack = Array.prototype.slice.call(arguments, 1);
  var item;
  var key;
  while (stack.length) {
    item = stack.shift();
    for (key in item) {
      if (item.hasOwnProperty(key)) {
        if (Object.prototype.toString.call(result[key]) === '[object Object]') {
          result[key] = assignment(result[key], item[key]);
        } else {
          result[key] = item[key];
        }
      }
    }
  }
  return result;
}

var assignment_1 = assignment;

var lowercase = function lowercase (string) {
  return typeof string === 'string' ? string.toLowerCase() : string;
};

function toMap (list) {
  return list.reduce(asKey, {});
}

function asKey (accumulator, item) {
  accumulator[item] = true;
  return accumulator;
}

var toMap_1 = toMap;

var uris = ['background', 'base', 'cite', 'href', 'longdesc', 'src', 'usemap'];

var attributes = {
  uris: toMap_1(uris) // attributes that have an href and hence need to be sanitized
};

var voids = ['area', 'br', 'col', 'hr', 'img', 'wbr', 'input', 'base', 'basefont', 'link', 'meta'];

var elements = {
  voids: toMap_1(voids)
};

var rstart = /^<\s*([\w:-]+)((?:\s+[\w:-]+(?:\s*=\s*(?:(?:"[^"]*")|(?:'[^']*')|[^>\s]+))?)*)\s*(\/?)\s*>/;
var rend = /^<\s*\/\s*([\w:-]+)[^>]*>/;
var rattrs = /([\w:-]+)(?:\s*=\s*(?:(?:"((?:[^"])*)")|(?:'((?:[^'])*)')|([^>\s]+)))?/g;
var rtag = /^</;
var rtagend = /^<\s*\//;

function createStack () {
  var stack = [];
  stack.lastItem = function lastItem () {
    return stack[stack.length - 1];
  };
  return stack;
}

function parser (html, handler) {
  var stack = createStack();
  var last = html;
  var chars;

  while (html) {
    parsePart();
  }
  parseEndTag(); // clean up any remaining tags

  function parsePart () {
    chars = true;
    parseTag();

    var same = html === last;
    last = html;

    if (same) { // discard, because it's invalid
      html = '';
    }
  }

  function parseTag () {
    if (html.substr(0, 4) === '<!--') { // comments
      parseComment();
    } else if (rtagend.test(html)) {
      parseEdge(rend, parseEndTag);
    } else if (rtag.test(html)) {
      parseEdge(rstart, parseStartTag);
    }
    parseTagDecode();
  }

  function parseEdge (regex, parser) {
    var match = html.match(regex);
    if (match) {
      html = html.substring(match[0].length);
      match[0].replace(regex, parser);
      chars = false;
    }
  }

  function parseComment () {
    var index = html.indexOf('-->');
    if (index >= 0) {
      if (handler.comment) {
        handler.comment(html.substring(4, index));
      }
      html = html.substring(index + 3);
      chars = false;
    }
  }

  function parseTagDecode () {
    if (!chars) {
      return;
    }
    var text;
    var index = html.indexOf('<');
    if (index >= 0) {
      text = html.substring(0, index);
      html = html.substring(index);
    } else {
      text = html;
      html = '';
    }
    if (handler.chars) {
      handler.chars(text);
    }
  }

  function parseStartTag (tag, tagName, rest, unary) {
    var attrs = {};
    var low = lowercase(tagName);
    var u = elements.voids[low] || !!unary;

    rest.replace(rattrs, attrReplacer);

    if (!u) {
      stack.push(low);
    }
    if (handler.start) {
      handler.start(low, attrs, u);
    }

    function attrReplacer (match, name, doubleQuotedValue, singleQuotedValue, unquotedValue) {
      if (doubleQuotedValue === void 0 && singleQuotedValue === void 0 && unquotedValue === void 0) {
        attrs[name] = void 0; // attribute is like <button disabled></button>
      } else {
        attrs[name] = she.decode(doubleQuotedValue || singleQuotedValue || unquotedValue || '');
      }
    }
  }

  function parseEndTag (tag, tagName) {
    var i;
    var pos = 0;
    var low = lowercase(tagName);
    if (low) {
      for (pos = stack.length - 1; pos >= 0; pos--) {
        if (stack[pos] === low) {
          break; // find the closest opened tag of the same type
        }
      }
    }
    if (pos >= 0) {
      for (i = stack.length - 1; i >= pos; i--) {
        if (handler.end) { // close all the open elements, up the stack
          handler.end(stack[i]);
        }
      }
      stack.length = pos;
    }
  }
}

var parser_1 = parser;

function sanitizer (buffer, options) {
  var context;
  var o = options || {};

  reset();

  return {
    start: start,
    end: end,
    chars: chars
  };

  function out (value) {
    buffer.push(value);
  }

  function start (tag, attrs, unary) {
    var low = lowercase(tag);

    if (context.ignoring) {
      ignore(low); return;
    }
    if ((o.allowedTags || []).indexOf(low) === -1) {
      ignore(low); return;
    }
    if (o.filter && !o.filter({ tag: low, attrs: attrs })) {
      ignore(low); return;
    }

    out('<');
    out(low);
    Object.keys(attrs).forEach(parse);
    out(unary ? '/>' : '>');

    function parse (key) {
      var value = attrs[key];
      var classesOk = (o.allowedClasses || {})[low] || [];
      var attrsOk = (o.allowedAttributes || {})[low] || [];
      var valid;
      var lkey = lowercase(key);
      if (lkey === 'class' && attrsOk.indexOf(lkey) === -1) {
        value = value.split(' ').filter(isValidClass).join(' ').trim();
        valid = value.length;
      } else {
        valid = attrsOk.indexOf(lkey) !== -1 && (attributes.uris[lkey] !== true || testUrl(value));
      }
      if (valid) {
        out(' ');
        out(key);
        if (typeof value === 'string') {
          out('="');
          out(she.encode(value));
          out('"');
        }
      }
      function isValidClass (className) {
        return classesOk && classesOk.indexOf(className) !== -1;
      }
    }
  }

  function end (tag) {
    var low = lowercase(tag);
    var allowed = (o.allowedTags || []).indexOf(low) !== -1;
    if (allowed) {
      if (context.ignoring === false) {
        out('</');
        out(low);
        out('>');
      } else {
        unignore(low);
      }
    } else {
      unignore(low);
    }
  }

  function testUrl (text) {
    var start = text[0];
    if (start === '#' || start === '/') {
      return true;
    }
    var colon = text.indexOf(':');
    if (colon === -1) {
      return true;
    }
    var questionmark = text.indexOf('?');
    if (questionmark !== -1 && colon > questionmark) {
      return true;
    }
    var hash = text.indexOf('#');
    if (hash !== -1 && colon > hash) {
      return true;
    }
    return o.allowedSchemes.some(matches);

    function matches (scheme) {
      return text.indexOf(scheme + ':') === 0;
    }
  }

  function chars (text) {
    if (context.ignoring === false) {
      out(o.transformText ? o.transformText(text) : text);
    }
  }

  function ignore (tag) {
    if (elements.voids[tag]) {
      return;
    }
    if (context.ignoring === false) {
      context = { ignoring: tag, depth: 1 };
    } else if (context.ignoring === tag) {
      context.depth++;
    }
  }

  function unignore (tag) {
    if (context.ignoring === tag) {
      if (--context.depth <= 0) {
        reset();
      }
    }
  }

  function reset () {
    context = { ignoring: false, depth: 0 };
  }
}

var sanitizer_1 = sanitizer;

var defaults = {
  allowedAttributes: {
    a: ['href', 'name', 'target', 'title', 'aria-label'],
    iframe: ['allowfullscreen', 'frameborder', 'src'],
    img: ['src', 'alt', 'title', 'aria-label']
  },
  allowedClasses: {},
  allowedSchemes: ['http', 'https', 'mailto'],
  allowedTags: [
    'a', 'abbr', 'article', 'b', 'blockquote', 'br', 'caption', 'code', 'del', 'details', 'div', 'em',
    'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'hr', 'i', 'img', 'ins', 'kbd', 'li', 'main', 'mark',
    'ol', 'p', 'pre', 'section', 'span', 'strike', 'strong', 'sub', 'summary', 'sup', 'table',
    'tbody', 'td', 'th', 'thead', 'tr', 'u', 'ul'
  ],
  filter: null
};

var defaults_1 = defaults;

function insane (html, options, strict) {
  var buffer = [];
  var configuration = strict === true ? options : assignment_1({}, defaults_1, options);
  var handler = sanitizer_1(buffer, configuration);

  parser_1(html, handler);

  return buffer.join('');
}

insane.defaults = defaults_1;
var insane_1 = insane;

const renderMarkdown = content => insane_1(t(content), {
  allowedAttributes: {
    a: ['href', 'name', 'target'],
    img: ['src'],
  },
  allowedSchemes: ['http', 'https', 'mailto'],
  allowedTags: ['a', 'b', 'br', 'code', 'em', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'hr', 'i', 'img', 'kbd', 'li', 'ol', 'strike', 'strong', 'sub', 'sup', 'u', 'ul'],
});

const Content = (props) => {
  const { content, classes, } = props;
  return (h("div", { class: clsx("hr-content hr-content-before", classes), innerHTML: renderMarkdown(content) }));
};

const hrFormCss = ":host{display:block}.hr-form-container{display:none}.hr-form-container-visible{display:block}";

const HappyreactForm = /*@__PURE__*/ proxyCustomElement(class HappyreactForm extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.close = createEvent(this, "close", 7);
    this.classes = undefined;
    this.mode = 'flow';
    this.state = {
      valid: false,
      dirty: false,
      values: {},
      errors: {
        _count: 0
      }
    };
    this.response = {
      status: 'loading',
      data: null
    };
    this.request = {
      status: 'idle',
      data: null
    };
    this.isVisible = false;
    this.props = null;
  }
  handleSend() {
    this.request.status = 'loading';
    saveResponse({
      token: this.props.token,
      reactionId: this.props.reactionId,
      url: this.props.url,
      values: this.state.values,
      resources: this.props.resources,
      count: this.props.count
    })
      .then((response) => {
      this.request = {
        status: 'success',
        data: response
      };
    });
  }
  handleValidate(value, field, state) {
    for (const validation of field.validations) {
      switch (validation.type) {
        case 'REQUIRED':
          if (!value) {
            const message = `${field.label} is required`;
            state.errors[field.id] = message;
            throw new Error(message);
          }
      }
    }
    this.state = state;
  }
  _countErrors(state) {
    return Object.keys(state.errors).filter((key) => {
      if (key.includes('_')) {
        return false;
      }
      return !!state.errors[key];
    }).length;
  }
  handleSubmit(event) {
    event.preventDefault();
    const state = structuredClone(this.state);
    for (const field of this.response.data.fields) {
      try {
        this.handleValidate(state.values[field.id], field, state);
      }
      catch (cause) {
        state.errors._count = this._countErrors(state);
      }
      finally {
        this.state = state;
      }
    }
    if (state.errors._count === 0) {
      this.handleSend();
    }
  }
  handleInput(event, field) {
    //@ts-expect-error
    const value = event.target.value;
    const state = structuredClone(this.state);
    state.dirty = true;
    state.values[field.id] = value;
    try {
      this.handleValidate(value, field, state);
      state.errors[field.id] = null;
    }
    catch (cause) {
      state.errors[field.id] = cause.message;
    }
    state.errors._count = this._countErrors(state);
    this.state = state;
  }
  /**
   * onReaction event handler
   * @param reaction
   */
  onCloseHandler() {
    this.close.emit();
    this.isVisible = false;
  }
  showHandler(event) {
    this.props = event.detail;
    this.state = {
      valid: false,
      dirty: false,
      values: {},
      errors: {
        _count: 0
      }
    };
    if (this.response.status === 'idle') {
      this.response = {
        status: 'loading',
        data: null
      };
    }
    fetchForm({
      token: this.props.token,
      reactionId: this.props.reactionId,
      url: this.props.url
    })
      .then((response) => {
      this.response = {
        status: 'success',
        data: response
      };
      this.request = {
        status: 'idle',
        data: null
      };
      this.isVisible = true;
    })
      .catch((cause) => {
      if (cause.message.includes('Cannot find form for reaction')) {
        this.response = {
          status: 'idle',
          data: null
        };
        return;
      }
      this.response = {
        status: 'error',
        data: cause.message
      };
    });
  }
  handleClick(event) {
    if (this.host.isSameNode(event.target)) {
      if (this.mode === 'modal') {
        this.onCloseHandler();
      }
    }
  }
  _getInput(field, getClass) {
    switch (field.type) {
      case 'TEXT':
        return h(TextInput, { id: field.id, key: field.id, onInput: (event) => this.handleInput(event, field), name: field.id, label: field.label, hint: field.hint, error: this.state.errors[field.id], getClass: getClass });
      case 'TEXTAREA':
        return h(TextareaInput, { key: field.id, onInput: (event) => this.handleInput(event, field), name: field.id, label: field.label, hint: field.hint, error: this.state.errors[field.id], getClass: getClass });
      case 'CONTENT':
        return h(ContentInput, { key: field.id, onInput: (event) => this.handleInput(event, field), name: field.id, label: field.label, hint: field.hint, error: this.state.errors[field.id] });
      default:
        throw new Error('Cannot match field type');
    }
  }
  render() {
    const getClass = createGetClass(this.classes);
    return this.isVisible ? (h(Host, { class: clsx('root', getClass('root')), onClick: this.handleClick.bind(this) }, h("div", { class: clsx("hr-form-container", this.isVisible && "hr-form-container-visible", getClass('container'), getClass('containerVisible')) }, h(Fetcher, { status: this.response.status }, this.request.status === 'success' ? (h(Success, { icon: this.request.data.icon, content: this.request.data.content, getClass: getClass })) : this.response.status === 'success' && (h(Fragment, null, this.mode === 'modal' && (h("div", { class: clsx('hr-close hr-close-top', getClass('close'), getClass('closeTop')) }, h("button", { onClick: this.onCloseHandler.bind(this), type: "button", class: clsx('hr-close-top-button', getClass('closeTopButton')) }, "X"))), this.response.data.content.before && (h(Content, { content: this.response.data.content.before, classes: clsx(getClass('content'), getClass('contentBefore')) })), h("form", { class: clsx("hr-form", getClass('form')), onSubmit: this.handleSubmit.bind(this) }, this.state.errors._count > 0 && (h("output", { class: clsx("hr-form-header", getClass('formHeader')), htmlFor: this.response.data.fields.map(field => field.id).join(' '), name: "errors" }, this.state.errors._count, " errors found:", h("ul", { class: clsx("hr-form-errors-list", getClass('formErrorsList')) }, this.response.data.fields.map((field) => this.state.errors[field.id] ? (h("li", { class: clsx("hr-form-error-item", getClass('formErrorItem')) }, this.state.errors[field.id])) : null)))), this.response.data.fields.map((field) => {
      return this._getInput(field, getClass);
    }), h("div", { class: clsx("hr-footer", getClass('footer')) }, this.request.status === 'idle' && (h("button", { type: "submit", class: clsx("hr-submit hr-submit-idle", getClass('submit'), getClass('submitIdle')) }, "Send")), this.request.status === 'loading' && (h("button", { type: "submit", class: clsx("hr-submit hr-submit-loading", getClass('submit'), getClass('submitIdle')), disabled: true }, "Saving...")))), this.response.data.content.after && (h(Content, { content: this.response.data.content.after, classes: clsx(getClass('content'), getClass('contentAfter')) })), this.mode === 'modal' && (h("div", { class: clsx('hr-close hr-close-bottom', getClass('close'), getClass('closeBottom')) }, h("button", { onClick: this.onCloseHandler.bind(this), type: "button", class: clsx('hr-close-bottom-button', getClass('closeBottomButton')) }, "Close"))))))))) : null;
  }
  get host() { return this; }
  static get style() { return hrFormCss; }
}, [0, "hr-form", {
    "classes": [16],
    "mode": [1],
    "state": [32],
    "response": [32],
    "request": [32],
    "isVisible": [32],
    "props": [32]
  }, [[0, "show", "showHandler"]]]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["hr-form"];
  components.forEach(tagName => { switch (tagName) {
    case "hr-form":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, HappyreactForm);
      }
      break;
  } });
}

const HrForm = HappyreactForm;
const defineCustomElement = defineCustomElement$1;

export { HrForm, defineCustomElement };

//# sourceMappingURL=hr-form.js.map